@@ -20,11 +20,6 @@
 
 #include "lookup.h"
 
-#include <arpa/inet.h>
-#include <arpa/nameser.h>
-#include <validator/resolver.h>
-#include <validator/validator.h>
-
 #include "QDNSItemModel.h"
 
 int
@@ -57,6 +52,18 @@ Lookup::setTypeText(const QString &label)
     m_queryButton->setText(label);
 }
 
+void val_qdebug(struct val_log *logp, int level, const char *buf)
+{
+    qDebug() << level << " -- " << buf;
+}
+
+static QList<QPair<int, QString> > val_log_strings;
+void val_collect_logs(struct val_log *logp, int level, const char *buf)
+{
+    val_log_strings.push_back(QPair<int, QString>(level, buf));
+}
+
+
 Lookup::Lookup(QWidget *parent)
     : QWidget(parent), found(false), m_queryType(ns_t_a)
 {
@@ -212,6 +219,9 @@ Lookup::Lookup(QWidget *parent)
     setLayout(vlayout);
     resize(QSize(800,400));
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+
+    //val_log_add_cb(NULL, 99, &val_qdebug);
+    val_log_add_cb(NULL, 99, &val_collect_logs);
 }
 
 QSize Lookup::sizeHint()
@@ -234,6 +244,7 @@ Lookup::dolookup()
     char printbuf[4096];
     int columns = 4;
 
+    val_log_strings.clear();
     m_answers->clear();
     m_answers->setColumnCount(columns);
     QStringList headers;
@@ -250,14 +261,19 @@ Lookup::dolookup()
 
     // do something with the results
     if (ret <= 0) {
-        m_answers->appendRow(new QStandardItem(p_val_status(val_status)));
-        
+        QStandardItem *answers = new QStandardItem("Results");
+        m_answers->appendRow(answers);
+        answers->appendRow(new QStandardItem("No Answer Data"));
+        m_answerView->setExpanded(answers->index(), true);
+
         if (!val_istrusted(val_status)) {
             // untrusted error for host
         }
  	if (!val_istrusted(val_status)) {
             // untrusted for ip address
         }
+
+        setSecurityStatus(val_status);
     } else {
 
         ns_msg          handle;
@@ -348,46 +364,10 @@ Lookup::dolookup()
                 rrnum++;
             }
         }
-
-        //
-        // Set the security results into the display
-        //
-        QStandardItem *security = new QStandardItem("Security");
-        m_answers->appendRow(security);
         
-        if (val_isvalidated(val_status)) {
-            m_securityStatus = 
-                new QStandardItem("Status: Validated");
-            m_answers->setSecurityStatus(m_securityStatus,
-                                         QDNSItemModel::validated);
-            m_resultsIcon->setPixmap(m_validated);
-#ifndef BROKENBACKGROUND
-            m_answerView->setStyleSheet("QTreeView { background-color: #96ff96; }");
-#endif
-        } else if (val_istrusted(val_status)) {
-            m_securityStatus = 
-                new QStandardItem("Status: Trusted");
-            m_answers->setSecurityStatus(m_securityStatus,
-                                         QDNSItemModel::trusted);
-            m_resultsIcon->setPixmap(m_trusted);
-#ifndef BROKENBACKGROUND
-            m_answerView->setStyleSheet("QTreeView { background-color: #ffff96; }");
-#endif
-        } else {
-            m_securityStatus = new QStandardItem("Status: Bogus");
-            m_answers->setSecurityStatus(m_securityStatus,
-                                         QDNSItemModel::bad);
-            m_resultsIcon->setPixmap(m_bad);
-#ifndef BROKENBACKGROUND
-            m_answerView->setStyleSheet("QTreeView { background-color: #ff9696; }");
-#endif
-        }
-
-        security->appendRow(m_securityStatus);
-        security->appendRow(new QStandardItem(QString("code: ") + QString(p_val_status(val_status))));
+        setSecurityStatus(val_status);
 
         m_answerView->setExpanded(results->index(), true);
-        m_answerView->setExpanded(security->index(), true);
         m_answerView->setExpanded(sections[ns_s_an]->index(), true);
         if (theRealAnswer) {
             m_answerView->setExpanded(theRealAnswer->index(), true);
@@ -411,6 +391,101 @@ Lookup::dolookup()
     unbusy();
 }
 
+void Lookup::setSecurityStatus(int val_status) {
+    //
+    // Set the security results into the display
+    //
+    QStandardItem *security = new QStandardItem("Security");
+    m_answers->appendRow(security);
+
+    if (val_isvalidated(val_status)) {
+        m_securityStatus =
+            new QStandardItem("Status: Validated");
+        m_answers->setSecurityStatus(m_securityStatus,
+                                     QDNSItemModel::validated);
+        m_resultsIcon->setPixmap(m_validated);
+    #ifndef BROKENBACKGROUND
+        m_answerView->setStyleSheet("QTreeView { background-color: #96ff96; }");
+    #endif
+    } else if (val_istrusted(val_status)) {
+        m_securityStatus =
+            new QStandardItem("Status: Trusted");
+        m_answers->setSecurityStatus(m_securityStatus,
+                                     QDNSItemModel::trusted);
+        m_resultsIcon->setPixmap(m_trusted);
+    #ifndef BROKENBACKGROUND
+        m_answerView->setStyleSheet("QTreeView { background-color: #ffff96; }");
+    #endif
+    } else {
+        m_securityStatus = new QStandardItem("Status: Bogus");
+        m_answers->setSecurityStatus(m_securityStatus,
+                                     QDNSItemModel::bad);
+        m_resultsIcon->setPixmap(m_bad);
+    #ifndef BROKENBACKGROUND
+        m_answerView->setStyleSheet("QTreeView { background-color: #ff9696; }");
+    #endif
+    }
+
+    security->appendRow(m_securityStatus);
+    security->appendRow(new QStandardItem(QString("code: ") + QString(p_val_status(val_status))));
+
+    QStandardItem *logs = new QStandardItem("Logs");
+    m_answers->appendRow(logs);
+    QStandardItem *interesting = new QStandardItem("Interesting");
+    logs->appendRow(interesting);
+
+    QStandardItem *allLogs = new QStandardItem("All");
+    logs->appendRow(allLogs);
+
+    // Interesting log engine
+    QRegExp keepit("([^:]+): +(.*)(looking for.*DNSKEY|looking for|Verified a RRSIG|Could not link|BOGUS|FAILURE|PINSECURE|Bogus|Cannot show|is provably insecure|matches|key.*is trusted|ending.*chain)(.*)");
+    QRegExp logParser("([^ ]+) +(.*)");
+
+    QString lastInterestingString;
+    QList<QPair<int, QString> >::iterator logEnd = val_log_strings.end();
+
+    for(QList<QPair<int, QString> >::iterator start = val_log_strings.begin();
+        start != logEnd; start++) {
+        QList<QStandardItem *> newRow;
+        int level = (*start).first;
+        QString valLog = (*start).second;
+
+        logParser.indexIn(valLog);
+        QString dateNTime = logParser.cap(1);
+        QString logText   = logParser.cap(2);
+        newRow.push_back(new QStandardItem(dateNTime));
+        newRow.push_back(new QStandardItem(QString().number(level)));
+        newRow.push_back(new QStandardItem());
+        newRow.push_back(new QStandardItem(logText));
+
+        //log->setColumnCount(newRow.count());
+
+        allLogs->appendRow(newRow);
+
+        // look for interesting logs
+        if (keepit.indexIn(valLog) <= 0)
+            continue;
+
+        QList<QStandardItem *> interestingLog;
+        QString interestingResults = keepit.cap(2) + keepit.cap(3) + keepit.cap(4);
+
+        if (interestingResults == lastInterestingString)
+            continue;
+
+        interestingLog.push_back(new QStandardItem(dateNTime));
+        interestingLog.push_back(new QStandardItem(QString().number(level)));
+        interestingLog.push_back(new QStandardItem());
+        interestingLog.push_back(new QStandardItem(interestingResults));
+
+        interesting->appendRow(interestingLog);
+
+        lastInterestingString = interestingResults;
+    }
+
+    m_answerView->setExpanded(security->index(), true);
+    m_answerView->setExpanded(logs->index(), true);
+}
+
 void Lookup::unbusy() {
     setCursor(Qt::ArrowCursor);
     gobutton->setEnabled(true);
