@@ -13,6 +13,11 @@
 #include <QtCore/QSize>
 #include <QtCore/QSignalMapper>
 
+#include <arpa/inet.h>
+#include <arpa/nameser.h>
+#include <validator/resolver.h>
+#include <validator/validator.h>
+
 #include "QDNSItemModel.h"
 
 class Lookup : public QWidget
@@ -30,6 +35,10 @@ class Lookup : public QWidget
     QSize sizeHint();
     void entryTextChanged(const QString &newtext);
 
+    void setSecurityStatus(int val_status);
+
+
+
   private:
     QLineEdit          *lookupline;
     QPushButton        *gobutton;
